package dao

import (
	"errors"
	"fmt"
	"github.com/gogf/gf/util/gconv"
	nebula "github.com/vesoft-inc/nebula-go/v3"
	"strconv"
	"time"
)

const (
	ADDRESS  = "10.9.33.129" // 开发机IP
	PORT     = 9669          // 端口号
	USERNAME = "root"        // 用户名
)

/*
封装增删改查操作，为了区分默认值和业务值，以及避免数据类型改变引起的数据迁移。
所有整型统一用字符串类型表示。只有时间类型用int64表示
*/
// Initialize logger
var (
	log     = nebula.DefaultLogger{}
	Session *nebula.Session
)

// EdgeVisit 访问关系边类型
type EdgeVisit struct {
	Proto     string // 协议
	EventTime int64  // 时间
}

// EdgeAttack 攻击关系边类型
type EdgeAttack struct {
	Name         string // 告警名称
	Proto        string // 协议
	APPProto     string // 应用层协议
	Tool         string // 黑客工具
	Phase        string // 攻击阶段
	Result       string // 攻击结果
	Tag          string // 威胁类型
	Severity     string // 威胁等级
	AttackerPort string // 攻击者端口
	VictimPort   string // 受害者端口
	EventTime    int64  // 告警时间
}

// TagAsset 资产tag
type TagAsset struct {
	Name     string // 资产名称
	Type     string // 资产类型
	Location string // 地理位置
}

// TagDefault 默认Tag，只有一个时间戳
type TagDefault struct {
	EventTime int64 // 添加时间
}

// Init 初始化客户端
func Init() error {
	hostAddress := nebula.HostAddress{Host: ADDRESS, Port: PORT}
	hostList := []nebula.HostAddress{hostAddress}
	// Create configs for connection pool using default values
	poolConfig := nebula.GetDefaultConf()
	// Initialize connection pool
	pool, err := nebula.NewConnectionPool(hostList, poolConfig, log)
	if err != nil {
		log.Error(fmt.Sprintf("Fail to initialize the connection pool, host: %s, port: %d, %s", ADDRESS, PORT, err.Error()))
	}
	// Create session
	Session, err = pool.GetSession(USERNAME, "")
	if err != nil || Session == nil {
		log.Error(fmt.Sprintf("Fail to create a new session from connection pool, username: %s, password: %s, %s",
			USERNAME, "", err.Error()))
	}
	err = InitSpace(Session, "demo")
	if err != nil {
		return err
	}
	err = CreatTag(Session)
	if err != nil {
		return err
	}
	err = CreatEdgeType(Session)
	if err != nil {
		return err
	}
	return nil
}

// InitSpace 初始化space，VID使用string类型，以IP为主
func InitSpace(session *nebula.Session, spaceName string) error {
	createSchema := "CREATE SPACE IF NOT EXISTS " + spaceName + "(vid_type=FIXED_STRING(65)); " +
		"USE " + spaceName + ";"
	resultSet, err := session.Execute(createSchema)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", createSchema, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	return nil
}

// CreatTag 创建Tag
func CreatTag(session *nebula.Session) error {
	createAssetTag := "CREATE TAG IF NOT EXISTS asset(name string, type string, location string);"
	createDefaultTag := "CREATE TAG IF NOT EXISTS default(`event_time` int64);"
	creatTag := createAssetTag + createDefaultTag
	resultSet, err := session.Execute(creatTag)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", creatTag, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	return nil
}

// CreatEdgeType 创建边类型
func CreatEdgeType(session *nebula.Session) error {
	createAttackEdge := "CREATE EDGE IF NOT EXISTS visit(proto string,`event_time` int64);"
	createVisitEdge := "CREATE EDGE IF NOT EXISTS attack(" +
		"name string,proto string,app_proto string," +
		"tool string,attacker_port string,victim_port " +
		"string,phase string,result string,`tag` string," +
		"severity string,`event_time` int64);"
	creatEdge := createAttackEdge + createVisitEdge
	resultSet, err := session.Execute(creatEdge)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", creatEdge, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	return nil
}

// InsertVisitEdge 插入互访关系的边
func InsertVisitEdge(session *nebula.Session, edgeVisit EdgeVisit, vidStart string, vidEnd string) error {
	times := time.Now().UnixNano() / 1e6
	// TODO 多重边查询怎么搞,时序怎么搞,可以考虑时间戳作为rank
	insertVisitEdge := "INSERT EDGE visit(`proto`, `event_time`) VALUES " +
		"\"" + vidStart + "\"" + "->" + "\"" + vidEnd + "\"" + ":(" +
		"\"" + edgeVisit.Proto + "\"" + "," + gconv.String(edgeVisit.EventTime) + ");"
	resultSet, err := session.Execute(insertVisitEdge)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", insertVisitEdge, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	fmt.Println("InsertAttackEdge=====", time.Now().UnixNano()/1e6-times)
	return nil
}

// InsertAttackEdge 插入告警类型的边 event_time作为rank
func InsertAttackEdge(session *nebula.Session, edgeAttack EdgeAttack, vidStart string, vidEnd string) error {
	times := time.Now().UnixNano() / 1e6
	insertAttackEdge := "INSERT EDGE attack(" +
		"`name`, `proto`, `app_proto`, " +
		"`tool`, `phase`, `result`, `tag`, " +
		"`severity`, `attacker_port`, `victim_port`, `event_time`) VALUES " +
		"\"" + vidStart + "\"" + "->" + "\"" + vidEnd + "\"" +
		"@" + strconv.FormatInt(edgeAttack.EventTime, 10) + ":(" +
		"\"" + edgeAttack.Name + "\"" + "," +
		"\"" + edgeAttack.Proto + "\"" + "," +
		"\"" + edgeAttack.APPProto + "\"" + "," +
		"\"" + edgeAttack.Tool + "\"" + "," +
		"\"" + edgeAttack.Phase + "\"" + "," +
		"\"" + edgeAttack.Result + "\"" + "," +
		"\"" + edgeAttack.Tag + "\"" + "," +
		"\"" + edgeAttack.Severity + "\"" + "," +
		"\"" + edgeAttack.AttackerPort + "\"" + "," +
		"\"" + edgeAttack.VictimPort + "\"" + "," +
		gconv.String(edgeAttack.EventTime) + ");"
	resultSet, err := session.Execute(insertAttackEdge)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", insertAttackEdge, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	fmt.Println("InsertAttackEdge=====", time.Now().UnixNano()/1e6-times)
	return nil
}

// InsertDefaultVertex  插入default tag的点
func InsertDefaultVertex(session *nebula.Session, vid string, tag TagDefault) error {
	times := time.Now().UnixNano() / 1e6
	insertVertex := "INSERT VERTEX default (`event_time`) VALUES " +
		"\"" + vid + "\"" + ":(" +
		strconv.FormatInt(tag.EventTime, 10) + ");"
	resultSet, err := session.Execute(insertVertex)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", insertVertex, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	fmt.Println("InsertDefaultVertex=====", time.Now().UnixNano()/1e6-times)
	return nil
}

// InsertAssetTagVertex  插入Asset tag的点
func InsertAssetTagVertex(session *nebula.Session, vid string, tag TagAsset) error {
	times := time.Now().UnixNano() / 1e6
	insertVertex := "INSERT VERTEX asset (name, type, location) VALUES " +
		"\"" + vid + "\"" + ":(" +
		"\"" + tag.Name + "\"" + "," +
		"\"" + tag.Type + "\"" + "," +
		"\"" + tag.Location + "\"" + ");"
	resultSet, err := session.Execute(insertVertex)
	if err != nil {
		log.Error(err.Error())
		return err
	}
	if !resultSet.IsSucceed() {
		log.Error(fmt.Sprintf("%s, ErrorCode: %v, ErrorMsg: %s", insertVertex, resultSet.GetErrorCode(), resultSet.GetErrorMsg()))
		return errors.New("nebula session exec failed")
	}
	fmt.Println("InsertAssetTagVertex=====", time.Now().UnixNano()/1e6-times)
	return nil
}
