package main

import (
	"bufio"
	"flag"
	"fmt"
	"io"
	"io/ioutil"
	"log"
	"nebula/dao"
	"os"
	"runtime"
	"runtime/pprof"
	"time"

	"github.com/bitly/go-simplejson"
	"github.com/gogf/gf/util/gconv"
)

var cpuprofile = flag.String("cpuprofile", "cpu.prof", "write cpu profile to `file`")
var memprofile = flag.String("memprofile", "mem.prof", "write memory profile to `file`")

func main() {
	s := time.Now().Unix()
	if *cpuprofile != "" {
		f, err := os.Create(*cpuprofile)
		if err != nil {
			log.Fatal("could not create CPU profile: ", err)
		}
		defer f.Close() // error handling omitted for example
		if err := pprof.StartCPUProfile(f); err != nil {
			log.Fatal("could not start CPU profile: ", err)
		}
		defer pprof.StopCPUProfile()
	}
	// ... rest of the program ...
	if *memprofile != "" {
		f, err := os.Create(*memprofile)
		if err != nil {
			log.Fatal("could not create memory profile: ", err)
		}
		defer f.Close() // error handling omitted for example
		runtime.GC()    // get up-to-date statistics
		if err := pprof.WriteHeapProfile(f); err != nil {
			log.Fatal("could not write memory profile: ", err)
		}
	}
	err := dao.Init()
	if err != nil {
		panic(err)
	}
	//time.Sleep(10 * time.Second)
	var done = make(chan struct{}, 1)
	var recordsChannel = make(chan string, 100000) // 缓存
	go func() {
		done <- dealFileRecord(recordsChannel)
	}()
	go func() {
		inputFileRecord(recordsChannel)
	}()
	err = dao.InsertDefaultVertex(dao.Session, "10", dao.TagDefault{
		EventTime: 1662918202281,
	})
	err = dao.InsertDefaultVertex(dao.Session, "15", dao.TagDefault{
		EventTime: 1662918202998,
	})
	if err != nil {
		panic(err)
	}
	err = dao.InsertVisitEdge(dao.Session, dao.EdgeVisit{Proto: "TCP"}, "10", "15")
	if err != nil {
		panic(err)
	}
	err = dao.InsertAttackEdge(dao.Session, dao.EdgeAttack{
		EventTime: 1662918202281,
	}, "10", "15")
	if err != nil {
		panic(err)
	}
	err = dao.InsertAssetTagVertex(dao.Session, "10", dao.TagAsset{Name: "test"})
	if err != nil {
		panic(err)
	}
	<-done
	fmt.Println(time.Now().Unix() - s)
}

func inputFileRecord(recordChannel chan string) {
	// 测试数据目录
	filePath := "./data"
	var files []string // 目标文件
	var count = 0
	fs, err := ioutil.ReadDir(filePath)
	if err != nil {
		panic(err)
	}
	// 只读取一级文件
	for _, fi := range fs {
		if !fi.IsDir() {
			fullName := filePath + "/" + fi.Name()
			files = append(files, fullName)
		}
	}
	for _, filePath := range files {
		f, err := os.Open(filePath)
		if err != nil {
			continue
		}
		reader := bufio.NewReader(f)
		for {
			readString, err := reader.ReadString('\n')
			if err != nil {
				if err == io.EOF {
					break
				} else {
					continue
				}
			}
			recordChannel <- readString
			count++
		}
		// 关闭
		err = f.Close()
		if err != nil {
			continue
		}
	}
}

func dealFileRecord(recordChannel chan string) struct{} {
	ticker := time.NewTicker(time.Second * 5)
	for {
		select {
		case record := <-recordChannel:
			jsonObj, err := simplejson.NewJson([]byte(record))
			if err != nil {
				panic(err)
			}
			// 从告警数据中获取点
			edge := GetAvailEdge(jsonObj)
			attackerTag := dao.TagDefault{
				EventTime: jsonObj.Get("timestamp").MustInt64(),
			}
			attackerVID := jsonObj.Get("attacker").MustString()
			victimTag := dao.TagDefault{
				EventTime: jsonObj.Get("timestamp").MustInt64(),
			}
			victimVID := jsonObj.Get("victim").MustString()
			err = dao.InsertDefaultVertex(dao.Session, attackerVID, attackerTag)
			if err != nil {
				panic(err)
			}
			err = dao.InsertDefaultVertex(dao.Session, victimVID, victimTag)
			if err != nil {
				panic(err)
			}
			err = dao.InsertAttackEdge(dao.Session, edge, attackerVID, victimVID)
			if err != nil {
				panic(err)
			}
			ticker.Reset(time.Second * 5)
		case <-ticker.C:
			return struct{}{}
		}

	}
}

// GetAvailEdge 获取两点一边,任何一个都不为空,防止悬挂边出现
func GetAvailEdge(jsonObj *simplejson.Json) dao.EdgeAttack {
	var edge dao.EdgeAttack
	edge.EventTime = jsonObj.Get("timestamp").MustInt64()
	edge.Name = jsonObj.Get("name").MustString()
	edge.Proto = jsonObj.Get("proto").MustString()
	edge.Tag = jsonObj.Get("tag").MustString()
	edge.Severity = gconv.String(jsonObj.Get("severity").MustInt())
	edge.Phase = jsonObj.Get("phase").MustString()
	edge.Tool = jsonObj.Get("tool").MustString()
	edge.APPProto = jsonObj.Get("app_proto").MustString()
	edge.Result = jsonObj.Get("result").MustString()
	edge.VictimPort = gconv.String(jsonObj.Get("victim_port").MustInt())
	edge.AttackerPort = gconv.String(jsonObj.Get("attacker_port").MustInt())
	return edge
}
